/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.fidentis.analyst.procrustes;

import cz.fidentis.analyst.Logger;
import cz.fidentis.analyst.feature.FeaturePoint;
import cz.fidentis.analyst.feature.api.IPosition;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import javax.vecmath.Vector3d;

/**
 * Utility functions that are used in multiple classes to compute
 *
 * @author Jakub Kolman
 */
public class ProcrustesUtils {

    /**
     * Sorts given list of feature points by their type
     *
     * @param featurePointList
     * @return sorted list of feature points
     */
    public static List<FeaturePoint> sortListByFeaturePointType(List<FeaturePoint> featurePointList) {
        Collections.sort(
                featurePointList, Comparator.comparingInt(fp -> fp.getFeaturePointType().getType())
        );
        return featurePointList;
    }

    /**
     * Moves point by given vector value.
     *
     * @param <T>
     * @param point
     * @param vector
     */
    public static <T extends IPosition> void movePoint(T point, Vector3d vector) {
        point.getPosition().x = point.getX() + vector.x;
        point.getPosition().y = point.getY() + vector.y;
        point.getPosition().z = point.getZ() + vector.z;
    }

    /**
     * Scales position of given point by multiplying its coordinates with given
     * scaleFactor.
     *
     * @param point
     * @param scaleFactor
     * @param <T>
     * @return
     */
    public static <T extends IPosition> T scalePointDistance(T point, double scaleFactor) {
        point.getPosition().x = point.getX() * scaleFactor;
        point.getPosition().y = point.getY() * scaleFactor;
        point.getPosition().z = point.getZ() * scaleFactor;
        return point;
    }

    /**
     * Checks if two feature point lists have the same types of feature points.
     * <p>
     * To use this method you need to supply ordered lists by Feature Point type
     * as parameters. Otherwise even if two lists contain the same feature
     * points it will return false.
     * <p>
     * Use sort function sortListByFeaturePointType on feature point lists to
     * get correct results.
     *
     * @param featurePointList1
     * @param featurePointList2
     * @return true if two sorted lists by the feature point type contain the
     * same feature point types
     */
    public static boolean checkFeaturePointsType(List<FeaturePoint> featurePointList1, List<FeaturePoint> featurePointList2) {
        for (int i = 0; i < featurePointList1.size(); i++) {
            if (featurePointList1.get(i).getFeaturePointType().getType() != featurePointList2.get(i).getFeaturePointType().getType()) {
                Logger.print(featurePointList1.get(i).getFeaturePointType().getName());
                Logger.print(featurePointList1.get(i).getFeaturePointType().getName());
                return false;
            }
        }
        return true;
    }
}
